/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <package_manager.h>

#include "model/app-manager-utils.h"
#include "utils/logger.h"

static bool __is_app_successful(app_manager_error_e err_code);
static bool __app_manager_app_info_cb(app_info_h app_info, void *user_data);

Eina_List *get_all_apps(void)
{
    Eina_List *app_list = NULL;
    app_manager_foreach_app_info(__app_manager_app_info_cb, &app_list);
    return app_list;
}

bool destroy_all_apps_list(Eina_List *app_list)
{
    Eina_List *it = NULL;
    app_info_h *app_info = NULL;
    EINA_LIST_FOREACH(app_list, it, app_info)
        if (__is_app_successful(app_info_destroy(*app_info)))
            return true;

    return false;
}

bool get_app_label(app_info_h *app_info, char **label)
{
    return __is_app_successful(app_info_get_label(*app_info, label));
}

bool get_app_icon(app_info_h *app_info, char **path)
{
    return __is_app_successful(app_info_get_icon(*app_info, path));
}

bool get_app_package(app_info_h *app_info, char **package)
{
    return __is_app_successful(app_info_get_package(*app_info, package));
}


static bool __is_app_successful(app_manager_error_e err_code)
{
#ifdef _DEBUG

    if (err_code == APP_MANAGER_ERROR_NONE) {
        return true;
    } else {
        const char *error_message = NULL;
        switch (err_code) {
        case APP_MANAGER_ERROR_INVALID_PARAMETER:
            error_message = "Invalid parameter";
            break;
        case APP_MANAGER_ERROR_OUT_OF_MEMORY:
            error_message = "Out of memory";
            break;
        case APP_MANAGER_ERROR_IO_ERROR:
            error_message = "Internal I/O error";
            break;
        case APP_MANAGER_ERROR_NO_SUCH_APP:
            error_message = "No such application";
            break;
        case APP_MANAGER_ERROR_DB_FAILED:
            error_message = "Database error";
            break;
        case APP_MANAGER_ERROR_INVALID_PACKAGE:
            error_message = "Invalid package name";
            break;
        case APP_MANAGER_ERROR_APP_NO_RUNNING:
            error_message = "App is not running";
            break;
        case APP_MANAGER_ERROR_REQUEST_FAILED:
            error_message = "Internal aul request error";
            break;
        case APP_MANAGER_ERROR_PERMISSION_DENIED:
            error_message = "Permission deniede";
            break;
        default:
            error_message = "Unknown error";
            break;
        }

        ERR("App manager %d: %s", err_code, error_message);
        return false;
    }
#else
    return err_code == APP_MANAGER_ERROR_NONE;
#endif /* _DEBUG */
}

static bool __app_manager_app_info_cb(app_info_h app_info, void *user_data)
{
    bool nodisplay;
    __is_app_successful(app_info_is_nodisplay(app_info, &nodisplay));
    if (!nodisplay) {
        Eina_List **app_list = user_data;
        app_info_h *clone = malloc(sizeof(app_info_h));
        __is_app_successful(app_info_clone(clone, app_info));
        *app_list = eina_list_append(*app_list, clone);
    }
    return true;
}

